@@ -3,9 +3,11 @@
 mod buffer;
 mod bytes;
 mod dict;
+mod fragment;
 mod list;
 
 pub use buffer::*;
 pub use bytes::*;
 pub use dict::*;
+pub use fragment::{orjson_fragmenttype_new, Fragment};
 pub use list::PyListIter;
